@@ -1,3 +1,17 @@
+#   Copyright  2020 Atos Spain SA. All rights reserved.
+ 
+#   This file is part of EASIER AI.
+ 
+#   EASIER AI is free software: you can redistribute it and/or modify it under the terms of Apache License, either version 2 of the License, or
+#   (at your option) any later version.
+ 
+#   THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT ANY WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING 
+#   BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT,
+#   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
+#   WHETHER IN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE 
+#   OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+#   See  LICENSE file for full license information  in the project root.
+
 import datetime as dt
 from elasticsearch import Elasticsearch
 import logging
@@ -14,6 +28,9 @@ import time
 import socket
 #from urllib3.exceptions import ConnectTimeoutError
 
+from ssl import create_default_context
+import certifi
+
 # This comment is necessary because pylint throws errors when using the elasticsearch API because it uses decorators.
 # pylint: disable=E1123
 # By default, queries will target to up to 10000 documents (maximum permitted by Elastic, without tweaking the legacy setup)
@@ -49,6 +66,23 @@ class Elasticsearchlib:
         self.es = _es
         ok = self.safe_ping()
         return ok, _es
+
+    def start_connection_secure(self, host, port, user, password, request_retries=3, total_retries=9):
+        """
+        :param request_retries: number of times a request will be retried before being dropped
+        :param total_retries: number of consecutive retries before dropping the connection and throwing an Exception
+        """
+        ssl_context = create_default_context(cafile=certifi.where())
+        _es = Elasticsearch(hosts=[{'host': host, 'port': port}],
+                            http_auth=('easier', 'easier-123.'),
+                            scheme="https",
+                            ssl_context=ssl_context)
+        self.request_retries = request_retries
+        self.total_retries = total_retries
+        self.current_total_retries = self.total_retries
+        self.es = _es
+        ok = self.safe_ping()
+        return ok, _es
         
     # Request functions
     def create_index(self, index, mapping=None):
@@ -152,6 +186,51 @@ class Elasticsearchlib:
         self.safe_clear_scroll(scroll_id)
         return True, output
 
+    def scrolled_query_generator(self, query, index, filter_path=None, query_size=QUERY_SIZE, max_size=MAX_RESPONSE_SIZE, time_index=None):
+        '''
+        : max_size parameter should be a multiple of query_size or the results won't have the expected size.
+        '''
+        output = []
+        if query_size > MAX_QUERY_SIZE:
+            self._logger.warning('Passing a greater query size ('+ str(query_size) +  ') than maximum permitted by elasticsearch. Setting value to ' + str(MAX_QUERY_SIZE))
+            query_size = MAX_QUERY_SIZE
+        ok, res = self.safe_search(index, query, filter_path, size=query_size)
+        if ok:
+            scroll_id = res['_scroll_id']
+            if (res['hits']['total']['value']):
+                output = _.concat(output, res['hits']['hits'])
+                if time_index:
+                    yield True, self.remove_duplicates_by_field(output, time_index)
+                else:
+                    yield True, output
+            if (res['hits']['total']['value']) > query_size and len(output) < max_size:
+                more = True
+                while more and len(output) < max_size:
+                    ok, res = self.safe_scroll(scroll_id, filter_path=filter_path)
+                    if ok:
+                        if _.has(res, 'hits.hits'):
+                            more = res['hits']['hits']
+                            output = []
+                            output = _.concat(output, res['hits']['hits'])
+                            if time_index:
+                                yield True,  self.remove_duplicates_by_field(output, time_index)
+                            else:
+                                yield True, output
+                        else:
+                            more = False
+
+                    else:
+                        self.safe_clear_scroll(scroll_id)
+                        yield False, []
+            # ToDo - performance and logging
+
+            ###
+        else:
+            self.safe_clear_scroll(scroll_id)
+            yield False, []
+        self.safe_clear_scroll(scroll_id)
+        yield True, output
+
     def get_entities_ids(self, index):
 
         filter_path = ['_scroll_id', 'took', 'hits.total', 'hits.hits._id']
@@ -277,16 +356,18 @@ class Elasticsearchlib:
         return True, res
     
     def safe_clear_scroll(self, scroll_id):
-        retries = self.request_retries
-        while(retries):
-            try:
-                self.es.clear_scroll(scroll_id=scroll_id)
-                retries = self.reset_current_retries()
-                return True
-            except Exception as ex:
-                self._logger.error("Couldn't clear scroll " + str(ex))
-                retries = self.check_retries(retries)
-                if(retries == 0): return False
+        if scroll_id:
+            retries = self.request_retries
+            while(retries):
+                try:
+                    self.es.clear_scroll(scroll_id=scroll_id)
+                    retries = self.reset_current_retries()
+                    return True
+                except Exception as ex:
+                    self._logger.error("Couldn't clear scroll " + str(ex))
+                    retries = self.check_retries(retries)
+                    if(retries == 0): return False
+        return True
 
     def safe_create_index(self, index, mapping):
         retries = self.request_retries
